/*	NAVZUP - simple backup solution for MS Dynamics Navision ZUP-files
 *	Copyright (C) 2011 David Rupprechter, rupprechter@dotlike.net, www.dotlike.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "gui.h"
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <string.h>
#include <tchar.h>
#include <gtkmm/stock.h>
#include <gtkmm/stockid.h>
#include <gtkmm/toolbar.h>
#include <gtkmm.h>
#include <glibmm/thread.h>
#include <gtkmm/window.h>
#include <gtkmm/box.h>
#include <gtkmm/button.h>
#include <gtkmm/messagedialog.h>
#include "file.h"
#include "map.h"
#include <glib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <windows.h>
#include <iostream>
#include <fstream>
#include <shlobj.h>    // for SHGetFolderPath
#include <commdlg.h>   // for GetSaveFileName
#include <shlwapi.h>	// pathappend

#include <atlconv.h>

#pragma comment(lib, "Shlwapi.lib") // lib for PathAppend

char timeStr[ 100 ] = "\0";
struct stat buf;
Fle zfiles;

bool set = false;

TCHAR zuPath[MAX_PATH];
TCHAR szPath[MAX_PATH];
TCHAR path2zup[MAX_PATH];
char buffer[MAX_PATH];
char bufferrest[MAX_PATH];
char bufferpath[MAX_PATH];
string customdir;
char bakfle[280]; // backup/restore-file info
char destfle[280]; // backup/restore-file info
char del_file_restore[280];
char del_file_backup[280];

string zpfile;

// integer for liststore numbering
int x = 1;
int y = 1;


std::wstring fileinfo;
std::string filedate;
std::string filenfo;

std::wstring fileinfobak;
std::string filedatebak;
std::string filenfobak;

M_map backupmap;
M_map restoremap;



void Winz::on_button_folder_clicked()
{
	Gtk::FileSelection dialog("Please choose an alternative ZUP-file directory");
	
	//Prevent the user from selecting a file.
	dialog.get_file_list()->get_parent()->hide();

	dialog.set_transient_for(*this);
	
	int result = dialog.run();

  //Handle the response:
  switch(result)
  {
    case(Gtk::RESPONSE_OK):
    {
	  // set selected folder for later use
	  ofstream zupfolder;
	  zupfolder.open( "zupfolder.cfg", ios::trunc);
	  zupfolder << dialog.get_filename() << "\\";
	  zupfolder.close();
	  set = true;
      break;
    }

    case(Gtk::RESPONSE_CANCEL):
    {
	  set = true;
      break;
    }

    default:
    {
      break;
    }
  }
}



Winz::Winz()
: m_bak("NAV-files"),
  m_restore("Backups")
{
	set_title("NavZUP v1.2");
	set_border_width(15);
	set_default_size(800, 200);

	Gtk::Button *m_info = new Gtk::Button();
	Gtk::Button *m_del = new Gtk::Button();
	Gtk::Button *m_refresh = new Gtk::Button();
	Gtk::Button *m_back = new Gtk::Button();
	Gtk::Button *m_rest = new Gtk::Button();

	Gtk::Widget *infoimg = Gtk::manage(new Gtk::Image(Gtk::Stock::ABOUT, Gtk::ICON_SIZE_BUTTON));
	Gtk::Widget *delimg = Gtk::manage(new Gtk::Image(Gtk::Stock::CANCEL, Gtk::ICON_SIZE_BUTTON));
	Gtk::Widget *restimg = Gtk::manage(new Gtk::Image(Gtk::Stock::GO_BACK, Gtk::ICON_SIZE_BUTTON));
	Gtk::Widget *bakimg = Gtk::manage(new Gtk::Image(Gtk::Stock::GO_FORWARD, Gtk::ICON_SIZE_BUTTON));
	Gtk::Widget *refreshimg = Gtk::manage(new Gtk::Image(Gtk::Stock::REFRESH, Gtk::ICON_SIZE_BUTTON));

	
	m_info->add(*infoimg);
	m_refresh->add(*refreshimg);
	m_rest->add(*restimg);
	m_back->add(*bakimg);
	m_del->add(*delimg);

	m_info->set_tooltip_text( "copyright and license information" );
	m_refresh->set_tooltip_text( "refresh views" );
	m_rest->set_tooltip_text( "restore selected ZUP-file to NAV-folder" );
	m_back->set_tooltip_text( "backup selected ZUP-file to NavZUP-backup-folder" );
	m_del->set_tooltip_text( "delete selected ZUP-file" );

	m_ScrolledWindow_bak.add(m_TreeView_bak);
	m_ScrolledWindow_restore.add(m_TreeView_restore);

	// scrolling for liststore
	m_ScrolledWindow_bak.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	m_ScrolledWindow_restore.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	//Create the Tree model:
	m_refTreeModel_bak = Gtk::ListStore::create(m_Columns_bak);
	m_TreeView_bak.set_model(m_refTreeModel_bak);

	m_refTreeModel_restore = Gtk::ListStore::create(m_Columns_restore);
	m_TreeView_restore.set_model(m_refTreeModel_restore);

	//Add the TreeView's view columns:
	//This number will be shown with the default numeric formatting.
	m_TreeView_bak.append_column("ID\t", m_Columns_bak.m_col_id);
	m_TreeView_bak.append_column("file\t\t\t\t", m_Columns_bak.m_col_name);
	m_TreeView_bak.append_column("modified\t\t", m_Columns_bak.m_col_date);

	m_TreeView_restore.append_column("ID\t", m_Columns_restore.m_col_id);
	m_TreeView_restore.append_column("file\t\t\t\t", m_Columns_restore.m_col_name);
	m_TreeView_restore.append_column("modified\t\t", m_Columns_restore.m_col_date);


	m_refresh->signal_clicked().connect( sigc::mem_fun(*this, &Winz::refresh_on_button_clicked) );
	m_del->signal_clicked().connect( sigc::mem_fun(*this, &Winz::delete_on_button_clicked) );
	m_rest->signal_clicked().connect( sigc::mem_fun(*this, &Winz::restore_on_button_clicked) );
	m_back->signal_clicked().connect( sigc::mem_fun(*this, &Winz::backup_on_button_clicked) );
	m_info->signal_clicked().connect( sigc::mem_fun(*this, &Winz::info_on_button_clicked) );

	// put backup/restore in
	hbak.pack_start(m_ScrolledWindow_bak, Gtk::PACK_EXPAND_WIDGET);
	hrestore.pack_start(m_ScrolledWindow_restore, Gtk::PACK_EXPAND_WIDGET);

	// size request
	m_refresh->set_size_request(100, 40);
	m_del->set_size_request(100, 40);
	m_back->set_size_request(100, 40);
	m_rest->set_size_request(100, 40);
	m_info->set_size_request(100, 40);

	hbuttons.pack_start(*m_refresh, Gtk::PACK_EXPAND_PADDING);
	m_refresh->show();
	hbuttons.pack_start(*m_del, Gtk::PACK_EXPAND_PADDING);
	m_del->show();
	hbuttons.pack_start(*m_back, Gtk::PACK_EXPAND_PADDING);
	m_back->show();
	hbuttons.pack_start(*m_rest, Gtk::PACK_EXPAND_PADDING);
	m_restore.show();
	hbuttons.pack_start(*m_info, Gtk::PACK_EXPAND_PADDING);
	m_info->show();
	
	vBox.pack_start(hTopO, Gtk::PACK_SHRINK);
	add(vBox);

	hTopO.pack_start(m_bak, Gtk::PACK_SHRINK);
	hTopO.pack_end(m_restore, Gtk::PACK_SHRINK);
	
	vBox.pack_end(hTop, Gtk::PACK_EXPAND_WIDGET);
	hTop.set_spacing(20);
	hTop.pack_start(hbak, Gtk::PACK_EXPAND_WIDGET);
	hbak.show();
	hTop.pack_start(hbuttons, Gtk::PACK_SHRINK);
	hbuttons.show();
	hTop.pack_start(hrestore, Gtk::PACK_EXPAND_WIDGET);
	hrestore.show();

	hTop.show();
	vBox.show();

	show_all_children();

	Winz::refresh_on_button_clicked();

}

Winz::~Winz()
{
	hide();
}

void Winz::info_on_button_clicked()
{
  Gtk::MessageDialog dialog(*this, "NavZUP v1.2");
  dialog.set_secondary_text(
          "Copyright (C) 2011 David Rupprechter\ndavid.rupprechter@dotlike.net\nwww.dotlike.net\n\nThis program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.");
  dialog.run();
}



void Winz::refresh_on_button_clicked()
{
	// set = false
	set = false;

	// reset list-numbering
	x = 1; 
	y = 1;

	// clear map
	backupmap.clear();
	restoremap.clear();

	typedef Gtk::TreeModel::Children chld;

	chld childrenbak = m_refTreeModel_bak->children();
	chld childrenrest = m_refTreeModel_restore->children();
	
	m_refTreeModel_bak->clear();
	m_refTreeModel_restore->clear();
	
	// find zup files				
	if( zfiles.findzup() != 1 ) {
		MessageBox(HWND_DESKTOP,L"Failed to intialize ZUP files",L"NavZup Refresh",MB_ICONERROR);
	}

	if ( zfiles.findzup() == 1) { 
		if (backupmap.m_size() == 0 && set == false) {
			// no zup file found
			MessageBox(HWND_DESKTOP, L"NAV ZUP-File not found! Please set the directory manually"  , L"NavZUP Error" , MB_ICONERROR);
			
			Winz::on_button_folder_clicked();

			// reset list-numbering
			x = 1; 
			y = 1;

			// clear map
			backupmap.clear();
			restoremap.clear();
			
			zfiles.findzup();
		}
	}


	// NAVISION-Liststore
	for (multimap<wstring, string>::iterator iter=backupmap.begin(); iter!=backupmap.end(); iter++)
	{
		if (x == 1 && iter != backupmap.end() ) {
			wstring fname_sel = backupmap.mget_filename( iter );
			filenfo = backupmap.mget_fileinfo( iter );

			string filen(fname_sel.length(), ' ');
			copy( fname_sel.begin(), fname_sel.end(), filen.begin());

			Gtk::TreeModel::Row row = *(m_refTreeModel_bak->append());
			row[m_Columns_bak.m_col_id] = x++;
			row[m_Columns_bak.m_col_name] = filen;
			row[m_Columns_bak.m_col_date] = filenfo;
			}

		else {
				wstring fname_sel = backupmap.mget_filename( iter );
				filenfo = backupmap.mget_fileinfo( iter );

				string filen(fname_sel.length(), ' ');
				copy( fname_sel.begin(), fname_sel.end(), filen.begin());

				Gtk::TreeModel::Row row = *(m_refTreeModel_bak->append());
				row[m_Columns_bak.m_col_id] = x++;
				row[m_Columns_bak.m_col_name] = filen;
				row[m_Columns_bak.m_col_date] = filenfo;
		}
	}

	// NAVZUP.BAK LISTSTORE
	for (multimap<wstring, string>::iterator iter=restoremap.begin(); iter!=restoremap.end(); iter++)
	{
		if (y == 1 && iter != restoremap.end() ) {
			wstring fname_sel = restoremap.mget_filename( iter );
			filenfo = restoremap.mget_fileinfo( iter );

			string filen(fname_sel.length(), ' ');
			copy( fname_sel.begin(), fname_sel.end(), filen.begin());

			Gtk::TreeModel::Row row = *(m_refTreeModel_restore->append());
			row[m_Columns_bak.m_col_id] = y++;
			row[m_Columns_bak.m_col_name] = filen;
			row[m_Columns_bak.m_col_date] = filenfo;
		}

		else {
			wstring fname_sel = restoremap.mget_filename( iter );
			filenfo = restoremap.mget_fileinfo( iter );

			string filen(fname_sel.length(), ' ');
			copy( fname_sel.begin(), fname_sel.end(), filen.begin());

			Gtk::TreeModel::Row row = *(m_refTreeModel_restore->append());
			row[m_Columns_bak.m_col_id] = y++;
			row[m_Columns_bak.m_col_name] = filen;
			row[m_Columns_bak.m_col_date] = filenfo;
		}
	}
}

void Winz::backup_on_button_clicked()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView_bak.get_selection();
	Gtk::TreeModel::iterator iterbak = refTreeSelection->get_selected();
	
	if(iterbak) //If anything is selected
	{
		Gtk::TreeModel::Row row = *iterbak;
		
		Glib::ustring fbak = row[m_Columns_bak.m_col_name];

		bakfle[MAX_PATH] = '\0';
		destfle[MAX_PATH] = '\0';

		strcpy(bakfle, bufferpath);
		strcat(bakfle, fbak.c_str() );

		USES_CONVERSION;

		strcpy(bakfle, bufferpath);
		strcat(bakfle, fbak.c_str() );

		strcat(destfle, "navzup.bak\\");
		strcat(destfle, fbak.c_str() );

		zfiles.copyfile( A2CW(bakfle), A2CW(destfle) );
		
		// update list
		zfiles.findzup();
		Winz::refresh_on_button_clicked();
	}
	else {
		MessageBox(HWND_DESKTOP, L"Nothing selected? Please select a file from the Backup-list"  , L"NavZUP Error" , MB_ICONERROR);
	}	
}

void Winz::restore_on_button_clicked()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView_restore.get_selection();
	Gtk::TreeModel::iterator iterrest = refTreeSelection->get_selected();
	
	if(iterrest) //If anything is selected
	{
		Gtk::TreeModel::Row row = *iterrest;
		
		Glib::ustring frest = row[m_Columns_bak.m_col_name];


		strcpy(destfle, bufferpath );
		strcat(destfle, frest.c_str() );

		strcpy(bakfle, "navzup.bak\\");
		strcat(bakfle, frest.c_str() );
		USES_CONVERSION;

		zfiles.copyfile( A2CW(bakfle), A2CW(destfle) );


		// update list
		zfiles.findzup();
		Winz::refresh_on_button_clicked();		
	}
	else {
		MessageBox(HWND_DESKTOP, L"Nothing selected? Please select a file from the Restore-list"  , L"NavZUP Error" , MB_ICONERROR);
	}		
}

void Winz::delete_on_button_clicked()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView_bak.get_selection();
	Gtk::TreeModel::iterator iterbak = refTreeSelection->get_selected();
	
	if(iterbak) //If anything is selected
	{
		Gtk::TreeModel::Row row = *iterbak;
		Glib::ustring fbak = row[m_Columns_bak.m_col_name];

		strcpy(del_file_backup, bufferpath);
		strcat(del_file_backup, fbak.c_str() );

		USES_CONVERSION;
		zfiles.deletezup( A2CW(del_file_backup) );
		zfiles.findzup();
		Winz::refresh_on_button_clicked();
	}
	else {
		Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView_restore.get_selection();
		Gtk::TreeModel::iterator iterrest = refTreeSelection->get_selected();	
		
		if(iterrest) //If anything is selected
		{
			Gtk::TreeModel::Row row = *iterrest;
			Glib::ustring frest = row[m_Columns_bak.m_col_name];

			strcpy(del_file_restore, "navzup.bak\\");
			strcat(del_file_restore, frest.c_str() );

			USES_CONVERSION;
			zfiles.deletezup( A2CW(del_file_restore) );
			zfiles.findzup();
			Winz::refresh_on_button_clicked();
		}
	
	
	}		
}




int Fle::findzup() {
	customdir.clear();
	zpfile.clear();

	WIN32_FIND_DATA FindData;
	WIN32_FIND_DATA FindDataBak;
	HANDLE hFind, hFindBak;

	// if Navision custom zup-folder was set before...
 	ifstream zfile ("zupfolder.cfg", ifstream::in);
	if ( !zfile.fail() ) // file exsits
	{
		while (! zfile.eof() ) {
			getline ( zfile, zpfile );
		}
    
		zfile.close();
		
		// append *.zup to path
		customdir.clear();
		customdir.append( zpfile );

		wstring temp2(customdir.length(),L' ');
		std::copy(customdir.begin(), customdir.end(), temp2.begin() );
		_tcscpy( path2zup, temp2.c_str() );
		customdir.append( "*.zup" );

		wstring temp(customdir.length(),L' ');
		std::copy(customdir.begin(), customdir.end(), temp.begin());

		// if custom zup-dir set
		if (customdir.size() > 0 ) {
			_tcscpy( szPath, temp.c_str() );
		}
	}

	// if custom zup-dir not set
	else {  
		if ( SUCCEEDED( SHGetFolderPath( NULL, CSIDL_APPDATA, 
                                    NULL, 0, szPath ) ) )
		{
			SHGetFolderPath( NULL, CSIDdL_APPDATA, 
                                    NULL, 0, path2zup );

			PathAppend( szPath, TEXT("*.zup") ); // place where zup-files are located

		}
		else {
			MessageBox(HWND_DESKTOP, L"Cant set AppData directory!"  , L"NavZUP Error" , MB_ICONERROR);
		}
	
	}
	// search zup files in appdata dir
	hFind = FindFirstFileEx( szPath, FindExInfoStandard, &FindData, FindExSearchNameMatch, NULL , 0);

	
		SYSTEMTIME stUTC, stUTCbak, stLocalbak, stLocal;

		// nav-dir
		FileTimeToSystemTime(&FindData.ftLastAccessTime, &stUTC);
		SystemTimeToTzSpecificLocalTime(NULL, &stUTC, &stLocal);

		USES_CONVERSION; // for T2A

		// NAVISION - DIR

		// copy nav-zupdir to bufferpath
		strcpy(bufferpath, T2A ( path2zup ) );


		char Azup[MAX_PATH];	// for path
		char Azupfile[MAX_PATH];	// file

		char AFilename[MAX_PATH];
		string file;

		backupmap.clear();
		restoremap.clear();

		size_t len, lenafile;

		buffer[MAX_PATH] = '\0';
		Azup[MAX_PATH] = '\0';
		AFilename[MAX_PATH] = '\0';

		// copy found Filename to AFilename
		WideCharToMultiByte(CP_ACP, 0, FindData.cFileName, wcslen(FindData.cFileName)+1, AFilename , sizeof(AFilename), NULL, NULL);
		WideCharToMultiByte(CP_ACP, 0, path2zup, wcslen(path2zup)+1, Azup , sizeof(Azup), NULL, NULL); // conversion
		
		strcpy( buffer, Azup);
		strcat( buffer, AFilename);

		backupmap.clear();

		// if stating file is ok
		if(!stat( buffer , &buf)) {
			strftime(timeStr, sizeof(timeStr), "%d-%m-%y %X", localtime( &buf.st_mtime));
			// if stating ok, put into list
			fileinfo = (LPTSTR)FindData.cFileName;
			filedate = timeStr;
			if( wcslen(FindData.cFileName) > 0) {
				backupmap.m_insert( fileinfo , filedate );
			}
		}

		// NAVISION-FOLDER
		while ( FindNextFile ( hFind, &FindData ) != NULL) // while next file is found
		{
			Azup[MAX_PATH] = '\0';
			AFilename[MAX_PATH] = '\0';
			
			// put together folder and filename for stating file
			WideCharToMultiByte(CP_ACP, 0, FindData.cFileName, wcslen(FindData.cFileName)+1, AFilename , sizeof(AFilename), NULL, NULL);
			WideCharToMultiByte(CP_ACP, 0, path2zup, wcslen(path2zup)+1, Azup , sizeof(Azup), NULL, NULL);

			buffer[MAX_PATH] = '\0';
			strcpy( buffer, Azup);
			strcat( buffer, AFilename);

			if(!stat( buffer , &buf)) {
				strftime(timeStr, sizeof(timeStr), "%d-%m-%y %X", localtime( &buf.st_mtime));

				fileinfo = (LPTSTR)FindData.cFileName;
				filedate = timeStr;

				backupmap.m_insert( fileinfo , filedate );
			}		
		}

		if ( GetLastError() != ERROR_NO_MORE_FILES) { // if another error than no more files occurs
			//
		}
		FindClose(hFind);



		// bak-dir
		FileTimeToSystemTime(&FindDataBak.ftLastAccessTime, &stUTCbak);
		SystemTimeToTzSpecificLocalTime(NULL, &stUTCbak, &stLocalbak);

	
		// NAVZUP.BAK DIR
		// backup: put together folder and filename for stating file

		// search zup files in navzup-bak-dir
		hFindBak = FindFirstFileEx( L"navzup.bak\\*.zup", FindExInfoStandard, &FindDataBak, FindExSearchNameMatch, NULL , 0);

		if (hFindBak == INVALID_HANDLE_VALUE) { // error? print it...
			if ( GetLastError() != ERROR_ALREADY_EXISTS) {
				if ( CreateDirectory(L"navzup.bak", NULL) == NULL) {
				}
			}
		}

		WideCharToMultiByte(CP_ACP, 0, FindDataBak.cFileName, wcslen(FindDataBak.cFileName)+1, AFilename , sizeof(AFilename), NULL, NULL);

		bufferrest[MAX_PATH] = '\0';
		strcpy(bufferrest, "navzup.bak\\");
		strcat(bufferrest, AFilename);

		restoremap.clear();

		if(!stat( bufferrest , &buf)) {
			strftime(timeStr, sizeof(timeStr), "%d-%m-%y %X", localtime( &buf.st_mtime));

			// stating ok, put into list
			fileinfo = (LPTSTR)FindDataBak.cFileName;
			filedate = timeStr;
		
			restoremap.m_insert( fileinfo , filedate );
		}

		// NAVZUP.BAK FOLDER
		while ( FindNextFile ( hFindBak, &FindDataBak ) != NULL) // while next file is found
		{	// put file info into map?/...

		WideCharToMultiByte(CP_ACP, 0, FindDataBak.cFileName, wcslen(FindDataBak.cFileName)+1, AFilename , sizeof(AFilename), NULL, NULL);

			// put together folder and filename for stating file
			bufferrest[MAX_PATH] = '\0';
			strcpy(bufferrest, "navzup.bak\\");
			strcat(bufferrest, AFilename);
			
			if(!stat( bufferrest , &buf)) {
				strftime(timeStr, sizeof(timeStr), "%d-%m-%y %X", localtime( &buf.st_mtime));
				fileinfo = (LPTSTR)FindDataBak.cFileName;
				filedate = timeStr;

				restoremap.m_insert( fileinfo , filedate );
			}
		}

		if ( GetLastError() != ERROR_NO_MORE_FILES) { // if another error than no more files occurs
			//
		}
		FindClose(hFindBak);

	return 1;
}




Fle::Fle() {

}

int Fle::deletezup( LPCWSTR zupfile ) {
	if(DeleteFile( zupfile ))
		{  }
	else {
		if ( GetLastError() == ERROR_FILE_NOT_FOUND) {
			MessageBox(HWND_DESKTOP, L"Error: file not found!"  , L"NavZUP" , MB_ICONERROR);
			return 2;		
		}

		if ( GetLastError() == ERROR_ACCESS_DENIED) {
			MessageBox(HWND_DESKTOP, L"Error: Access denied!"  , L"NavZUP" , MB_ICONERROR);
			return 2;
		}
		return 2;
	}

	return 1;
}

int Fle::copyfile( LPCWSTR source, LPCWSTR dest) {
	if(CopyFile( source , dest , FALSE ))
		{	}

	else
	{
	
		if ( GetLastError() == ERROR_FILE_NOT_FOUND) { // if another error than no more files occurs
			MessageBox(HWND_DESKTOP, L"Error: file not found!"  , L"NavZUP" , MB_ICONERROR);
			return 2;
		}
	
	}
	return 1;
}

Fle::~Fle() {

}